using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using ParticleSystem;
using ArcBallCamera;

namespace XNASimpleParticle
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // input
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;

        // camera
        SampleArcBallCamera myCamera;
        // Set the position of the camera in world space, for our view matrix.
        Vector3 cameraPosition = new Vector3(0.0f, 50.0f, 5000.0f);
        float aspectRatio;

        Matrix View;
        Matrix Projection;
        Model grid;

        Rain rain;
        Bubbles bubbles;
        LaserShield shield;
        LaserScanner scanner;
        PoisonGas gas;
        Colorful colorful;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;

            rain = new Rain(this);
            Components.Add(rain);

            bubbles = new Bubbles(this);
            Components.Add(bubbles);

            shield = new LaserShield(this);
            Components.Add(shield);

            scanner = new LaserScanner(this);
            Components.Add(scanner);

            gas = new PoisonGas(this);
            Components.Add(gas);

            colorful = new Colorful(this);
            Components.Add(colorful);

            shield.Visible = shield.Enabled = false;
            scanner.Visible = scanner.Enabled = false;
            rain.Visible = rain.Enabled = false;
            bubbles.Visible = bubbles.Enabled = false;
            //gas.Visible = gas.Enabled = false;
            colorful.Visible = colorful.Enabled = false;

            myCamera = new SampleArcBallCamera(SampleArcBallCameraMode.Free);
            myCamera.Target = new Vector3(0, 0, 0);
            myCamera.Distance = -50f;
            myCamera.OrbitUp(-0.1f);

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            grid = Content.Load<Model>("grid");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            float elapsedTime = (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);
            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space) &&
                    prevKeyboardState.IsKeyUp(Keys.Space))
            {                
                myCamera.Target = new Vector3(0, 0, 0);
                myCamera.Distance = -50f;
                //myCamera.OrbitUp(-0.1f);

                colorful.ResetSystem();
                gas.ResetSystem();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.N) &&
                    prevKeyboardState.IsKeyUp(Keys.N))
            {
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Left))
            {
                float dX = elapsedTime * 3.0f;
                myCamera.OrbitRight(dX);
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Right))
            {
                float dX = -elapsedTime * 3.0f;
                myCamera.OrbitRight(dX);
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Up))
            {
                float dY = elapsedTime * 3.0f;
                myCamera.OrbitUp(dY);
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Down))
            {
                float dY = -elapsedTime * 3.0f;
                myCamera.OrbitUp(dY);
            }
            else if (currentKeyboardState.IsKeyDown(Keys.PageUp))
            {
                myCamera.Distance -= elapsedTime * 4.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.PageDown))
            {
                myCamera.Distance += elapsedTime * 4.0f;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// Helper for drawing the background grid model.
        /// </summary>
        void DrawGrid(Matrix view, Matrix projection)
        {
            GraphicsDevice device = graphics.GraphicsDevice;

            device.RenderState.AlphaBlendEnable = false;
            device.RenderState.AlphaTestEnable = false;
            device.RenderState.DepthBufferEnable = true;

            device.SamplerStates[0].AddressU = TextureAddressMode.Wrap;
            device.SamplerStates[0].AddressV = TextureAddressMode.Wrap;

            foreach (ModelMesh mesh in grid.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.View = view;
                    effect.Projection = projection;
                }

                mesh.Draw();
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            //set our particle system's view and projection matrices before calling base.Draw
            //Matrix View = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            View = myCamera.ViewMatrix;
            aspectRatio = graphics.GraphicsDevice.Viewport.AspectRatio;
            Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 10000.0f);

            // Draw our background grid .
            DrawGrid(View, Projection);

            // Draw particles
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = false;
            graphics.GraphicsDevice.RenderState.PointSpriteEnable = false;
            graphics.GraphicsDevice.RenderState.DepthBufferWriteEnable = true;

            rain.View = View;
            rain.Projection = Projection;

            bubbles.View = View;
            bubbles.Projection = Projection;

            shield.View = View;
            shield.Projection = Projection;

            scanner.View = View;
            scanner.Projection = Projection;

            gas.View = View;
            gas.Projection = Projection;

            colorful.View = View;
            colorful.Projection = Projection;

            base.Draw(gameTime);
        }
    }
}
